@@ -31,7 +31,6 @@ use Moo;
 use Try::Tiny;
 
 use Lintian::Output qw(msg);
-use Lintian::Tags qw(tag);
 use Lintian::Util qw(copy_dir clean_env);
 
 with('Lintian::Check');
@@ -68,7 +67,8 @@ sub source {
                 push(@lang_templates, $basename);
                 my $fd = $path->open;
                 while (<$fd>) {
-                    tag 'untranslatable-debconf-templates', "$basename: $."
+                    $self->tag('untranslatable-debconf-templates',
+                        "$basename: $.")
                       if (m/^Description: (.+)/i and $1 !~/for internal use/);
                 }
                 close($fd);
@@ -77,9 +77,10 @@ sub source {
                 my $in_template = 0;
                 my $saw_tl_note = 0;
                 while (<$fd>) {
-                    tag 'translated-default-field', "$basename: $."
+                    $self->tag('translated-default-field', "$basename: $.")
                       if (m{^_Default(?:Choice)?: [^\[]*$}) && !$saw_tl_note;
-                    tag 'untranslatable-debconf-templates', "$basename: $."
+                    $self->tag('untranslatable-debconf-templates',
+                        "$basename: $.")
                       if (m/^Description: (.+)/i and $1 !~/for internal use/);
 
                     if (/^#/) {
@@ -115,7 +116,7 @@ sub source {
     #TODO: check whether all templates are named in TEMPLATES.pot
     if ($has_template) {
         if (not $debian_po_dir or not $debian_po_dir->is_dir) {
-            tag 'not-using-po-debconf';
+            $self->tag('not-using-po-debconf');
             return;
         }
     } else {
@@ -125,7 +126,8 @@ sub source {
     # If we got here, we're using po-debconf, so there shouldn't be any stray
     # language templates left over from debconf-mergetemplate.
     for (@lang_templates) {
-        tag 'stray-translated-debconf-templates', $_ unless /templates\.in$/;
+        $self->tag('stray-translated-debconf-templates', $_)
+          unless /templates\.in$/;
     }
 
     my $missing_files = 0;
@@ -140,20 +142,20 @@ sub source {
             next if $_ eq ''; #m,^\.\./, or
             my $po_path = $debian_dir->resolve_path($_);
             unless ($po_path and $po_path->is_file) {
-                tag 'missing-file-from-potfiles-in', $_;
+                $self->tag('missing-file-from-potfiles-in', $_);
                 $missing_files = 1;
             }
         }
         close($fd);
     } else {
-        tag 'missing-potfiles-in';
+        $self->tag('missing-potfiles-in');
         $missing_files = 1;
     }
     if (not $templ_pot_path or not $templ_pot_path->is_open_ok) {
         # We use is_open_ok here, because if it is present, we will
         # (have a subprocess) open it if the POTFILES.in file also
         # existed.
-        tag 'missing-templates-pot';
+        $self->tag('missing-templates-pot');
         $missing_files = 1;
     }
 
@@ -217,7 +219,7 @@ sub source {
         msg $_ for @messages;
 
         if ($error) {
-            tag 'invalid-potfiles-in';
+            $self->tag('invalid-potfiles-in');
             return;
         }
 
@@ -246,8 +248,7 @@ sub source {
             };
         };
 
-        tag 'newer-debconf-templates'
-          if length $error;
+        $self->tag('newer-debconf-templates') if length $error;
     }
 
     return unless $debian_po_dir;
@@ -255,7 +256,7 @@ sub source {
     for my $po_path ($debian_po_dir->children) {
         my $basename = $po_path->basename;
         next unless $basename =~ m/\.po$/;
-        tag 'misnamed-po-file', $po_path
+        $self->tag('misnamed-po-file', $po_path)
           unless ($basename =~ /^[a-z]{2,3}(_[A-Z]{2})?(?:\@[^\.]+)?\.po$/o);
         next unless $po_path->is_open_ok;
         local ($/) = "\n\n";
@@ -264,13 +265,13 @@ sub source {
         while (<$fd>) {
 
             if (/Language\-Team:.*debian-i18n\@lists\.debian\.org/i) {
-                tag 'debconf-translation-using-general-list', $basename;
+                $self->tag('debconf-translation-using-general-list',$basename);
             }
             last if m/^msgstr/m;
         }
         close($fd);
         unless ($_) {
-            tag 'invalid-po-file', $po_path;
+            $self->tag('invalid-po-file', $po_path);
             next;
         }
         s/"\n"//g;
@@ -278,7 +279,7 @@ sub source {
         if (m/charset=(.*?)\\n/) {
             $charset = ($1 eq 'CHARSET' ? '' : $1);
         }
-        tag 'unknown-encoding-in-po-file', $po_path
+        $self->tag('unknown-encoding-in-po-file', $po_path)
           unless length($charset);
 
         my $error;
@@ -301,15 +302,14 @@ sub source {
             };
         };
 
-        tag 'invalid-po-file', $po_path
-          if length $error;
+        $self->tag('invalid-po-file', $po_path) if length $error;
 
         if (!$full_translation && $stats =~ m/^\w+ \w+ \w+\.$/) {
             $full_translation = 1;
         }
     }
 
-    tag 'no-complete-debconf-translation' if !$full_translation;
+    $self->tag('no-complete-debconf-translation') if !$full_translation;
 
     return;
 }
