@@ -25,7 +25,6 @@ use Moo;
 
 use Lintian::Data;
 use Lintian::SlidingWindow;
-use Lintian::Tags qw(tag);
 use Lintian::Util qw(rstrip);
 
 with('Lintian::Check');
@@ -152,7 +151,7 @@ sub source {
     # (dpkg-source does as well) and skip all the tests if we then
     # can't read it.
     if ($rules->is_symlink) {
-        tag 'debian-rules-is-symlink';
+        $self->tag('debian-rules-is-symlink');
         return unless $rules->is_open_ok;
     }
 
@@ -167,7 +166,7 @@ sub source {
     # strict reading of Policy doesn't allow either, but they seem harmless.
     my $start = <$rules_fd>;
     $start //= q{};
-    tag 'debian-rules-not-a-makefile'
+    $self->tag('debian-rules-not-a-makefile')
       unless $start =~ m%^\#!\s*/usr/bin/make\s+-[re]?f[re]?\s*$%;
 
     # Holds which dependencies are required.  The keys in %needed and
@@ -199,7 +198,7 @@ sub source {
         }
         my $line = $_;
 
-        tag 'debian-rules-is-dh_make-template'
+        $self->tag('debian-rules-is-dh_make-template')
           if $line =~ m/dh_make generated override targets/;
 
         next if /^\s*\#/;
@@ -214,8 +213,8 @@ sub source {
                 $includes = 1;
             }
             if ($DEPRECATED_MAKEFILES->known($makefile)){
-                tag 'debian-rules-uses-deprecated-makefile', "line $.",
-                  $makefile;
+                $self->tag('debian-rules-uses-deprecated-makefile',
+                    "line $.",$makefile);
             }
         }
 
@@ -223,7 +222,7 @@ sub source {
         # deprecated.  It's a bit easier structurally to do this here than in
         # debhelper.
         if (/^\s*(?:export\s+)?DH_COMPAT\s*:?=/ && keys(%seen) == 0) {
-            tag 'debian-rules-sets-DH_COMPAT', "line $.";
+            $self->tag('debian-rules-sets-DH_COMPAT', "line $.");
         }
 
         # Check for problems that can occur anywhere in debian/rules.
@@ -232,12 +231,12 @@ sub source {
         ) {
             # Ignore "-C<dir>" (#671537)
             if (not $1 or $1 !~ m,^C,) {
-                tag 'debian-rules-ignores-make-clean-error', "line $.";
+                $self->tag('debian-rules-ignores-make-clean-error', "line $.");
             }
         }
 
         if (/^\s*(?:export\s+)?DEB_BUILD_OPTIONS\s*:?=/ && keys(%seen) == 0) {
-            tag 'debian-rules-sets-DEB_BUILD_OPTIONS', "line $.";
+            $self->tag('debian-rules-sets-DEB_BUILD_OPTIONS', "line $.");
         }
 
         if (
@@ -247,7 +246,8 @@ sub source {
             /x
             && keys(%seen) == 0
         ) {
-            tag 'debian-rules-sets-dpkg-architecture-variable', "$1 (line $.)";
+            $self->tag('debian-rules-sets-dpkg-architecture-variable',
+                "$1 (line $.)");
         }
 
         # check generic problem
@@ -255,9 +255,9 @@ sub source {
             my $badregex = $BAD_CONSTRUCT_IN_RULES->value($bad_construct);
             if ($line =~ m/$badregex/) {
                 if (defined($+{info})) {
-                    tag $bad_construct, $+{info}, "(line $.)";
+                    $self->tag($bad_construct, $+{info}, "(line $.)");
                 } else {
-                    tag $bad_construct, "line $.";
+                    $self->tag($bad_construct, "line $.");
                 }
             }
         }
@@ -268,7 +268,7 @@ sub source {
             # rather well.
             my ($var, $value) = ($1, $2);
             $variables{$var} = $value;
-            tag 'unnecessary-source-date-epoch-assignment', "(line $.)"
+            $self->tag('unnecessary-source-date-epoch-assignment', "(line $.)")
               if $var eq 'SOURCE_DATE_EPOCH'
               and not $build_all->implies(
                 'dpkg-dev (>= 1.18.8) | debhelper (>= 10.10)');
@@ -325,7 +325,7 @@ sub source {
 
         if (!$includes
             && m/dpkg-parsechangelog.*(?:Source|Version|Date|Timestamp)/s) {
-            tag 'debian-rules-parses-dpkg-parsechangelog', "(line $.)";
+            $self->tag('debian-rules-parses-dpkg-parsechangelog', "(line $.)");
         }
 
         if (!/^ifn?(?:eq|def)\s/ && m/^([^\s:][^:]*):+(.*)/s) {
@@ -405,8 +405,8 @@ sub source {
                     my $group = $debhelper_order{$command};
                     $debhelper_group{$package} ||= 0;
                     if ($group < $debhelper_group{$package}) {
-                        tag 'debian-rules-calls-debhelper-in-odd-order',
-                          $command, "(line $.)";
+                        $self->tag('debian-rules-calls-debhelper-in-odd-order',
+                            $command, "(line $.)");
                     } else {
                         $debhelper_group{$package} = $group;
                     }
@@ -422,9 +422,10 @@ sub source {
             unless ($seen{$target}) {
                 my $typerule = $POLICYRULES->value($target);
                 if($typerule eq 'required') {
-                    tag 'debian-rules-missing-required-target', $target;
+                    $self->tag('debian-rules-missing-required-target',$target);
                 } elsif ($typerule eq 'recommended_allindep') {
-                    tag 'debian-rules-missing-recommended-target', $target;
+                    $self->tag('debian-rules-missing-recommended-target',
+                        $target);
                 } else {
                     $typerule ||= '<N/A>';
                     croak(
@@ -447,13 +448,13 @@ sub source {
                 $nonempty = 1;
             }
         }
-        tag 'binary-arch-rules-but-pkg-is-arch-indep' if $nonempty;
+        $self->tag('binary-arch-rules-but-pkg-is-arch-indep') if $nonempty;
     }
 
     foreach my $cmd (qw(dh_clean dh_fixperms)) {
         foreach my $suffix ('', '-indep', '-arch') {
             my $line = $overridden{"$cmd$suffix"};
-            tag "override_$cmd-does-not-call-$cmd", "(line $line)"
+            $self->tag("override_$cmd-does-not-call-$cmd", "(line $line)")
               if $line
               and none { m/^\t\s*-?($cmd\b|\$\(overridden_command\))/ }
             @{$rules_per_target{"override_$cmd$suffix"}};
@@ -463,16 +464,17 @@ sub source {
     if (my $line = $overridden{'dh_auto_test'}) {
         my @lines = grep {
                   $_ !~ m{^\t\s*[\:\[]}
+              and $_ !~ m{^\s*$}
               and $_ !~ m{\bdh_auto_test\b}
               and $_
               !~ m{^\t\s*[-@]?(?:(?:/usr)?/bin/)?(?:cp|chmod|echo|ln|mv|mkdir|rm|test|true)}
         } @{$rules_per_target{'override_dh_auto_test'}};
-        tag 'override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
-          "(line $line)"
+        $self->tag('override_dh_auto_test-does-not-check-DEB_BUILD_OPTIONS',
+            "(line $line)")
           if @lines and none { m/(DEB_BUILD_OPTIONS|nocheck)/ } @conditionals;
     }
 
-    tag 'debian-rules-contains-unnecessary-get-orig-source-target'
+    $self->tag('debian-rules-contains-unnecessary-get-orig-source-target')
       if any { m/^\s+uscan\b/ } @{$rules_per_target{'get-orig-source'}};
 
     # Make sure that all the required build dependencies are there.  Don't
@@ -485,12 +487,13 @@ sub source {
         my $tag = $needed_clean{$package} || 'missing-build-dependency';
         unless ($build_regular->implies($package)) {
             if ($build_indep->implies($package)) {
-                tag 'clean-should-be-satisfied-by-build-depends', $package;
+                $self->tag('clean-should-be-satisfied-by-build-depends',
+                    $package);
             } else {
                 if ($tag eq 'missing-build-dependency') {
-                    tag $tag, $package if $package ne 'debhelper';
+                    $self->tag($tag, $package) if $package ne 'debhelper';
                 } else {
-                    tag $tag;
+                    $self->tag($tag);
                 }
             }
         }
@@ -501,9 +504,9 @@ sub source {
 
         unless ($noarch->implies($package)) {
             if ($tag eq 'missing-build-dependency') {
-                tag $tag, $package;
+                $self->tag($tag, $package);
             } else {
-                tag $tag;
+                $self->tag($tag);
             }
         }
     }
@@ -514,7 +517,7 @@ sub source {
     while ($block = $sfd->readwindow) {
         foreach my $tag ($BAD_MULTILINE_CONSTRUCT_IN_RULES->all) {
             my $regex = $BAD_MULTILINE_CONSTRUCT_IN_RULES->value($tag);
-            tag $tag if $block =~ m/$regex/;
+            $self->tag($tag) if $block =~ m/$regex/;
         }
     }
     close($rules_fd);
