@@ -253,6 +253,7 @@ sub runner {
 
     my $command
       = "cd $runpath; $ENV{'LINTIAN_FRONTEND'} $testcase->{lintian_command_line} $testcase->{subject}";
+    say $command;
     my ($output, $status) = capture_merged { system($command); };
     $status = ($status >> 8) & 255;
 
@@ -316,7 +317,7 @@ sub check_literal {
 
     # run a sed-script if it exists
     my $actual = "$runpath/literal.actual.parsed";
-    my $script = "$runpath/post_test";
+    my $script = "$runpath/post-test";
     if(-f $script) {
         sed_hook($script, $raw, $actual);
     } else {
@@ -340,7 +341,7 @@ sub check_tags {
 
     # run a sed-script if it exists
     my $actual = "$runpath/tags.actual.parsed";
-    my $sedscript = "$runpath/post_test";
+    my $sedscript = "$runpath/post-test";
     if(-f $sedscript) {
         sed_hook($sedscript, $raw, $actual);
     } else {
@@ -350,7 +351,7 @@ sub check_tags {
 
     # calibrate tags; may write to $actual
     my $calibrated = "$runpath/tags.specified.calibrated";
-    my $calscript = "$runpath/test_calibration";
+    my $calscript = "$runpath/test-calibration";
     if(-x $calscript) {
         calibrate($calscript, $actual, $expected, $calibrated);
     } else {
@@ -405,8 +406,9 @@ sub check_result {
             push(@errors, 'Tags do not match');
 
             @difflines = reverse sort @difflines;
-            my $tagdiff = join(NEWLINE, @difflines) . NEWLINE;
-            path("$runpath/tagdiff")->spew($tagdiff);
+            my $tagdiff;
+            $tagdiff .= $_ . NEWLINE for @difflines;
+            path("$runpath/tagdiff")->spew($tagdiff // EMPTY);
 
         } else {
             die "Unknown match strategy $testcase->{match_strategy}.";
